@@ -35,17 +35,16 @@ __all__ = (
 def create_bagasse_pelleting_system(ins, outs):
     bagasse, = ins
     bagasse_pellets, = outs
-    U401 = units.ConveyingBelt('U401', bagasse)
-    U402 = units.HammerMill('U402', U401-0)
-    U403 = units.DrumDryer('U403', 
-        (U402-0, 'dryer_air', 'dryer_natural_gas'), 
-        ('', '', 'dryer_emissions'),
+    U401 = units.HammerMill('U401', bagasse)
+    U402 = units.DrumDryer('U402', 
+        (U401-0, 'dryer_air', 'dryer_natural_gas'), 
+        ('', 'dryer_outlet_air', 'dryer_emissions'),
         moisture_content=0.15, split=0.,
     )
-    X401 = bst.ThermalOxidizer('X401', (U403-1, 'oxidizer_air'), 'oxidizer_emissions')
-    U404 = units.ScrewFeeder('U404', U403-0)
-    U405 = units.BagassePelletMill('U405', U404-0)
-    U406 = units.ConveyingBelt('U406', U405-0, bagasse_pellets)
+    # X401 = bst.ThermalOxidizer('X401', (U403-1, 'oxidizer_air'), 'oxidizer_emissions')
+    U403 = units.ScrewFeeder('U403', U402-0)
+    U404 = units.BagassePelletMill('U404', U403-0)
+    U405 = units.ConveyingBelt('U405', U404-0, bagasse_pellets)
 
 @SystemFactory(
     ID='feedstock_handling_sys',
@@ -61,7 +60,7 @@ def create_bagasse_pelleting_system(ins, outs):
               total_flow=333334.2,
               units='kg/hr',
               price=0.03455)],
-    outs=[dict(ID='shreaded_cane')]
+    outs=[dict(ID='shredded_cane')]
 )
 def create_feedstock_handling_system(ins, outs):
     sugarcane, = ins
@@ -90,7 +89,7 @@ def create_juicing_system_without_treatment(ins, outs, pellet_bagasse=None):
     
     imbibition_water = bst.Stream('imbibition_water',
                                   Water=87023.35, units='kg/hr',
-                                  T = 338.15)
+                                  T = 350.15)
     
     # Hydrolyze starch
     T201 = units.EnzymeTreatment('T201', (sugarcane, enzyme), T=323.15)  # T=50
@@ -112,11 +111,10 @@ def create_juicing_system_without_treatment(ins, outs, pellet_bagasse=None):
                                          Sucrose=0.04,
                                          Solids=1),
                               moisture_content=0.5)
+    U202 = units.ConveyingBelt('U202', U201-0, [''] if pellet_bagasse else [bagasse])
     
     if pellet_bagasse:
-        bagasse_pelleting_sys = create_bagasse_pelleting_system(None, U201-0, bagasse, mockup=True)
-    else:
-        U202 = units.ConveyingBelt('U202', U201-0, bagasse)
+        bagasse_pelleting_sys = create_bagasse_pelleting_system(None, ins=U202-0, outs=bagasse, mockup=True)
     
     # Mix in water
     M201 = units.Mixer('M201', ('', imbibition_water), 1-U201)
@@ -229,15 +227,15 @@ def create_juicing_system_up_to_clarification(ins, outs, pellet_bagasse=None):
     ### Process specifications ###
     
     # Specifications dependent on lipid cane flow rate
+    @T201.add_specification(run=True)
     def correct_flows():
         F_mass = T201.ins[0].F_mass
         # correct enzyme, lime, phosphoric acid, and imbibition water
+        enzyme.F_mass *= F_mass / 333334.2
         lime.imass['CaO', 'Water'] = 0.001 * F_mass * np.array([0.046, 0.954])
         H3PO4.imass['H3PO4', 'Water'] = 0.00025 * F_mass
         T201._run()
     
-    T201.specification = correct_flows
-    
     # Specifications within a system
     def correct_wash_water():
         P202._run()
@@ -301,8 +299,8 @@ def create_beer_distillation_system(ins, outs,
     P301 = units.Pump(IDs.get('Beer pump', 'P301'))
     
     # Beer column
-    x_bot = 3.910570816782338e-06
-    y_top = 0.2811210085806504
+    x_bot = 3.91e-06
+    y_top = 0.28
     D302 = units.BinaryDistillation(IDs.get('Beer column', 'D302'), P=2. * 101325,
                                     outs=(distilled_beer, ''),
                                 y_top=y_top, x_bot=x_bot, k=1.1, Rmin=0.001,
@@ -490,17 +488,10 @@ def create_ethanol_purification_system(ins, outs,
           dict(ID='evaporator_condensate'),
           dict(ID='vent')],
 )
-def create_sucrose_fermentation_system(ins, outs):
+def create_sucrose_fermentation_system(ins, outs, scrubber=True):
     screened_juice, = ins
     beer, evaporator_condensate, vent = outs
     
-    ### Streams ###
-    
-    # Fresh water
-    stripping_water = bst.Stream('stripping_water',
-                                 Water=26836,
-                                 units='kg/hr')
-    
     dilution_water = bst.Stream('dilution_water')
     
     ### Units ###
@@ -524,17 +515,23 @@ def create_sucrose_fermentation_system(ins, outs):
     
     def get_sugar_concentration():
         s = M301.outs[0]
-        return s.imass['Glucose', 'Sucrose'].sum() / s.F_mass
+        if 'Lipid' in s.chemicals:
+            ignored = float(s.imass['Lipid'])
+        else:
+            ignored = 0.
+        return s.imass['Glucose', 'Sucrose'].sum() / (s.F_mass - ignored)
     
     def sugar_concentration_at_fraction_evaporated(V):
         F301.V = V
         F301._run()
+        P306._run()
         M301._run()
         return F301.sugar_concentration - get_sugar_concentration()
     
     def get_dilution_water():
         F301.V = 0
         F301._run()
+        P306._run()
         M301._run()
         target = F301.sugar_concentration
         current = get_sugar_concentration()
@@ -543,14 +540,30 @@ def create_sucrose_fermentation_system(ins, outs):
     @M301.add_specification(run=False)
     def adjust_glucose_concentration():
         V_guess = F301.V
+        s_dilution_water = M301.ins[2]
+        s_dilution_water.empty()
         dilution_water = get_dilution_water()
-        if dilution_water < 0:
+        if dilution_water < 0.:
+            y0 = sugar_concentration_at_fraction_evaporated(0.)
+            y1 = sugar_concentration_at_fraction_evaporated(1.)
+            if y1 > 0.: 
+                S301.split = 0.05
+                S301._run()
+                y1 = sugar_concentration_at_fraction_evaporated(1.)
+                if y1 > 0.: 
+                    S301.split = 0.
+                    S301._run()
+                    y1 = sugar_concentration_at_fraction_evaporated(1.)
+                    if y1 > 0.:
+                        breakpoint()
+                        raise RuntimeError('cannot evaporate to target sugar concentration')
             F301.V = flx.IQ_interpolation(
                 sugar_concentration_at_fraction_evaporated,
                 0., 1., x=V_guess, ytol=1e-5
             )
         else:
-            M301.ins[2].imass['Water'] = dilution_water
+            s_dilution_water.imass['Water'] = dilution_water
+        S301.split = 0.10
             
     F301.sugar_concentration = 0.23 # wt. % sugar
     
@@ -558,28 +571,34 @@ def create_sucrose_fermentation_system(ins, outs):
     H301 = units.HXutility('H301', T=295.15)
     
     # Ethanol Production
-    R301 = units.Fermentation('R301', outs=('CO2', ''), tau=9, efficiency=0.90, N=4) 
+    R301 = units.Fermentation('R301', outs=(('CO2' if scrubber else vent), ''), tau=9, efficiency=0.90, N=4) 
     T301 = units.StorageTank('T301', tau=4, vessel_material='Carbon steel')
     T301.line = 'Beer tank'
     
-    stripping_water_over_vent = stripping_water.mol / 21202.490455845436
-    def update_stripping_water():
-        stripping_water, vent = D301.ins
-        stripping_water.mol[:] = stripping_water_over_vent * vent.F_mass
-    
-    D301 = units.VentScrubber('D301', ins=(stripping_water, R301-0), 
-                              outs=(vent, ''),
-                              gas=('CO2', 'O2'))
+    if scrubber:
+        stripping_water = bst.Stream('stripping_water',
+                                     Water=26836,
+                                     units='kg/hr')
+        stripping_water_over_vent = stripping_water.mol / 21202.490455845436
+        def update_stripping_water():
+            stripping_water, vent = D301.ins
+            stripping_water.mol[:] = stripping_water_over_vent * vent.F_mass
+        
+        D301 = units.VentScrubber('D301', ins=(stripping_water, R301-0), 
+                                  outs=(vent, ''),
+                                  gas=('CO2', 'O2'))
+        M302 = units.Mixer('M302', outs=beer)
     
     # Separate 99% of yeast
     C301 = units.SolidsCentrifuge('C301', 
+                                  outs=('', '' if scrubber else beer),
                                   split=(1-1e-6, 0.99, 1, 0.01),
                                   order=('Ethanol', 'Glucose', 'H3PO4', 'DryYeast'),
                                   solids=('DryYeast',))
     C301.split[:] = 1. - C301.split
+    if 'Lipid' in C301.chemicals: C301.isplit['Lipid'] = 0.
     
-    # Mix in Water
-    M302 = units.Mixer('M302', outs=beer)
+    S302 = units.FakeSplitter('S302', None, ('', 'Yeast'))
     
     # Yeast mixing
     T305 = units.MixTank('T305')
@@ -589,22 +608,22 @@ def create_sucrose_fermentation_system(ins, outs):
     P306 = units.Pump('P306')
     
     def adjust_yeast_recycle():
-        recycle, beer = C301.outs
-        feed, *_ = C301.ins
+        recycle, beer = S302.outs
+        feed, = S302.ins
         yeast = 0.1 * feed.F_mass
         m = C301.moisture_content
         recycle.imass['Yeast', 'Water'] = [yeast, yeast * m / (1 - m)]
         beer.mol = feed.mol - recycle.mol
         beer.mol[beer.mol < 0.] = 0.
         beer.T = recycle.T = feed.T
-    C301.specification = adjust_yeast_recycle
+    S302.specification = adjust_yeast_recycle
     
     ### Ethanol system set-up ###
     
     screened_juice-S301-1-F301-0-P306
     (S301-0, P306-0, dilution_water)-M301-H301
-    (H301-0, C301-0-T305-0)-R301-1-T301-0-C301
-    (C301-1, D301-1)-M302
+    (H301-0, C301-0-S302-0)-R301-1-T301-0-C301-0-S302
+    if scrubber: (C301-1, D301-1)-M302
     
 
 @SystemFactory(
@@ -645,8 +664,7 @@ def create_sucrose_to_ethanol_system(ins, outs):
           dict(ID='emissions'),
           dict(ID='ash_disposal')]
 )
-def create_sugarcane_to_ethanol_system(ins, outs,
-    ):
+def create_sugarcane_to_ethanol_system(ins, outs):
     s = f.stream
     u = f.unit
     
@@ -697,5 +715,17 @@ def create_sugarcane_to_ethanol_system(ins, outs,
     F301 = u.F301
     D303 = u.D303
     HXN = bst.HeatExchangerNetwork('HXN', units=[F301, D303])
+    
+    # if vinasse_to_wastewater:
+    #     plant_air = bst.Stream('plant_air', N2=83333, units='kg/hr')
+    #     ADP = bst.facilities.AirDistributionPackage('ADP', plant_air)
+    #     @ADP.add_specification(run=True)
+    #     def adjust_plant_air():
+    #         plant_air.imass['N2'] = 0.8 * feedstock_handling_sys.ins[0].F_mass
+            
+    #     wastewater_treatment_sys = bst.create_wastewater_treatment_system(
+    #         ins=[vinasse],
+    #         mockup=True,
+    #     )
         
         
